@@ -39,6 +39,7 @@
 typedef	long long	count_t;
 
 struct tstat;
+struct devtstat;
 struct sstat;
 struct netpertask;
 
@@ -51,11 +52,11 @@ struct netpertask;
 #define RRNETATOPD	0x0008
 #define RRACCTACTIVE	0x0010
 #define RRIOSTAT	0x0020
+#define RRDOCKSTAT	0x0040
 
 struct visualize {
 	char	(*show_samp)  (time_t, int,
-	                struct sstat *, struct tstat *, struct tstat **,
-			int, int, int, int, int, int, int, int, 
+	                struct devtstat *, struct sstat *,
 			int, unsigned int, char);
 	void	(*show_error) (const char *, ...);
 	void	(*show_end)   (void);
@@ -108,20 +109,20 @@ extern int		almostcrit;
 #define	IOSTAT		0x00000004
 #define	NETATOP		0x00000010
 #define	NETATOPD	0x00000020
+#define	DOCKSTAT	0x00000040
 
 /*
 ** in rawlog file, the four least significant bits 
 ** are moved to the per-sample flags and therefor dummy
 ** in the support flags of the general header
 */
-#define        RAWLOGNG        (ACCTACTIVE|IOSTAT|NETATOP|NETATOPD)
+#define	RAWLOGNG	(ACCTACTIVE|IOSTAT|NETATOP|NETATOPD)
 
 /*
 ** structure containing the start-addresses of functions for visualization
 */
 char		generic_samp (time_t, int,
-		            struct sstat *, struct tstat *, struct tstat **,
-		            int, int, int, int, int, int, int, int,
+		            struct devtstat *, struct sstat *,
 		            int, unsigned int, char);
 void		generic_error(const char *, ...);
 void		generic_end  (void);
@@ -148,6 +149,7 @@ int		compmem(const void *, const void *);
 int		compnet(const void *, const void *);
 int		compusr(const void *, const void *);
 int		compnam(const void *, const void *);
+int		compcon(const void *, const void *);
 
 int		cpucompar (const void *, const void *);
 int		diskcompar(const void *, const void *);
@@ -157,9 +159,9 @@ int		contcompar(const void *, const void *);
 
 count_t		subcount(count_t, count_t);
 void  		rawread(void);
-char		rawwrite(time_t, int, struct sstat *, struct tstat *,
-			struct tstat **, int, int, int, int, int, int,
-			int, int, int, unsigned int, char);
+char		rawwrite (time_t, int,
+		            struct devtstat *, struct sstat *,
+		            int, unsigned int, char);
 
 int 		numeric(char *);
 void		getalarm(int);
