@@ -50,10 +50,10 @@
 #include <errno.h>
 #include <time.h>
 #include <sys/resource.h>
-#include <signal.h>
 #include <sys/socket.h>
 #include <sys/mman.h>
 #include <sys/statvfs.h>
+#include <sys/wait.h>
 
 #include "atop.h"
 #include "photoproc.h"
@@ -65,6 +65,9 @@
 #define	RETRYMS		25	// timeout (millisec) to read account record
 #define	NORECINTERVAL	3600	// no-record-available interval (seconds)
 
+#define PACCTSEC	3	// timeout (sec) to retry switch on pacct
+#define POLLSEC		1	// timeout (sec) when NETLINK fails
+
 #define GCINTERVAL      60      // garbage collection interval (seconds)
 
 /*
@@ -97,33 +100,39 @@ static char		cleanup_and_go = 0;
 /*
 ** function prototypes
 */
-static int		awaitprocterm(int, int, int, char *, int *,
-					unsigned long *, unsigned long *);
-static int		createshadow(long);
-static int		pass2shadow(int, char *, int);
-static void		gcshadows(unsigned long *, unsigned long);
-static void		setcurrent(long);
-static int		acctsize(struct acct *);
-static void		cleanup(int);
+static int	awaitprocterm(int, int, int, char *, int *,
+				unsigned long *, unsigned long *);
+static int	swonpacct(int, char *);
+static int	createshadow(long);
+static int	pass2shadow(int, char *, int);
+static void	gcshadows(unsigned long *, unsigned long);
+static void	setcurrent(long);
+static int	acctsize(struct acct *);
+static void	cleanup(int);
 
-int			netlink_open(void);	// from netlink.c
+
+int		netlink_open(void);		// from netlink.c
+int		netlink_recv(int, int);		// from netlink.c
 
 
 int
 main(int argc, char *argv[])
 {
-	int		i, nfd, afd, sfd;
-	struct stat	dirstat;
-	struct rlimit	rlim;
-	FILE		*pidf;
+	int			i, nfd, afd, sfd;
+	int			parentpid;
+	struct stat		dirstat;
+	struct rlimit		rlim;
+	FILE			*pidf;
 
-	struct sembuf	semincr = {0, +1, SEM_UNDO};
+	struct sembuf		semincr = {0, +1, SEM_UNDO};
 
-	char		shadowdir[128], shadowpath[128];
-	char		accountpath[128];
-	unsigned long	oldshadow = 0, curshadow = 0;
-	int 		shadowbusy = 0;
-	time_t		gclast = time(0);
+	char			shadowdir[128], shadowpath[128];
+	char			accountpath[128];
+	unsigned long		oldshadow = 0, curshadow = 0;
+	int 			shadowbusy = 0;
+	time_t			gclast = time(0);
+
+	struct sigaction	sigcleanup;
 
 	/*
 	** argument passed?
@@ -251,26 +260,44 @@ main(int argc, char *argv[])
 		}
 	}
 
+	/*
+ 	** prepare cleanup signal handler
+	*/
+	memset(&sigcleanup, 0, sizeof sigcleanup);
+	sigcleanup.sa_handler	= cleanup;
+	sigemptyset(&sigcleanup.sa_mask);
+
 	/*
 	** daemonize this process
 	** i.e. be sure that the daemon is no session leader (any more)
 	** and get rid of a possible bad context that might have been
 	** inherited from ancestors
 	*/
+	parentpid = getpid();		// to be killed when initialized
+	(void) sigaction(SIGTERM, &sigcleanup, (struct sigaction *)0);
+
 	if ( fork() )			// implicitly switch to background
-		exit(0);		// continue in child process
+   	{
+		/*
+ 		** parent after forking first child:
+		** wait for signal 15 from child before terminating
+		** because systemd expects parent to terminate whenever
+		** service is up and running
+		*/
+		pause();		// wait for signal from child
+		exit(0);		// finish parent
+	}
 
 	setsid();			// become session leader to lose ctty
 
-	if ( fork() )			// continue in child process
+	if ( fork() )			// finish parent; continue in child
 		exit(0);		// --> no session leader, no ctty
 
-
 	getrlimit(RLIMIT_NOFILE, &rlim);
 
 	for (i=0; i < rlim.rlim_cur; i++)	// close all files, but
 	{
-		if (i != 2)			// close stderr later on
+		if (i != 2)			// do not close stderr 
 			close(i);
 	}
 
@@ -284,6 +311,7 @@ main(int argc, char *argv[])
 	if ( semop(semprv, &semincr, 1) == -1)
        	{
 		perror("cannot increment private semaphore");
+		kill(parentpid, SIGTERM);
 		exit(4);
 	}
 
@@ -298,6 +326,7 @@ main(int argc, char *argv[])
 	if ( (afd = creat(accountpath, 0600)) == -1)
        	{
 		perror(accountpath);
+		kill(parentpid, SIGTERM);
 		exit(5);
 	}
 
@@ -309,6 +338,7 @@ main(int argc, char *argv[])
 	if ( (afd = open(accountpath, O_RDONLY)) == -1)
        	{
 		perror(accountpath);
+		kill(parentpid, SIGTERM);
 		exit(5);
 	}
 
@@ -320,22 +350,13 @@ main(int argc, char *argv[])
 	if ( mkdir(shadowdir, 0755) == -1)
        	{
 		perror(shadowdir);
+		kill(parentpid, SIGTERM);
 		exit(5);
 	}
 
 	sfd = createshadow(curshadow);
 	setcurrent(curshadow);
 
-	/*
-	** switch on accounting
-	*/
-	if ( acct(accountpath) == -1)
-	{
-		perror("cannot switch on process accounting");
-		(void) unlink(accountpath);
-		exit(5);
-	}
-
 	/*
 	** open syslog interface 
 	*/
@@ -354,25 +375,31 @@ main(int argc, char *argv[])
 	*/
 	if ( (nfd = netlink_open()) == -1)
 	{
-		(void) acct(0);
 		(void) unlink(accountpath);
-		exit(6);
+		kill(parentpid, SIGTERM);
+		exit(5);
 	}
 
 	/*
-	** close stderr
-	** from now on, only generate messages via syslog
+	** switch on accounting - inital
 	*/
-	(void) close(2);
+	if ( swonpacct(afd, accountpath) == -1)
+	{
+		(void) unlink(accountpath);
+		kill(parentpid, SIGTERM);
+		exit(6);
+	}
+
+	syslog(LOG_INFO, "accounting to %s", accountpath);
 
 	/*
 	** signal handling
 	*/
 	(void) signal(SIGHUP, SIG_IGN);
 
-	(void) signal(SIGINT,  cleanup);
-	(void) signal(SIGQUIT, cleanup);
-	(void) signal(SIGTERM, cleanup);
+	(void) sigaction(SIGINT,  &sigcleanup, (struct sigaction *)0);
+	(void) sigaction(SIGQUIT, &sigcleanup, (struct sigaction *)0);
+	(void) sigaction(SIGTERM, &sigcleanup, (struct sigaction *)0);
 
 	/*
  	** create PID file
@@ -383,6 +410,11 @@ main(int argc, char *argv[])
 		fclose(pidf);
 	}
 
+	/*
+	** terminate parent: service  initialized
+	*/
+	kill(parentpid, SIGTERM);
+
 	/*
 	** main loop
 	*/
@@ -414,7 +446,7 @@ main(int argc, char *argv[])
 	/*
 	** cleanup and terminate
 	*/
-	(void) acct(0);			// disable process accounting
+	(void) acct((char *) 0);	// disable process accounting
 	(void) unlink(accountpath);	// remove source file
 
 	for (; oldshadow <= curshadow; oldshadow++)	// remove shadow files
@@ -430,14 +462,18 @@ main(int argc, char *argv[])
 
 	snprintf(shadowpath, sizeof shadowpath, "%s/%s/%s",
 			pacctdir, PACCTSHADOWD, PACCTSHADOWC);
-	(void) unlink(shadowpath);	// remove file 'current'
 
+	(void) unlink(shadowpath);	// remove file 'current'
 	(void) rmdir(shadowdir);	// remove shadow.d directory
 
 	if (cleanup_and_go)
 	{
 		syslog(LOG_NOTICE, "Terminated by signal %d\n", cleanup_and_go);
-		return cleanup_and_go + 128;
+
+		if (cleanup_and_go == SIGTERM)
+			return 0;
+		else
+			return cleanup_and_go + 128;
 	}
 	else
 	{
@@ -460,13 +496,13 @@ static int
 awaitprocterm(int nfd, int afd, int sfd, char *accountpath,
 	int *shadowbusyp, unsigned long *oldshadowp, unsigned long *curshadowp)
 {
-	static int			arecsize;
+	static int			arecsize, netlinkactive = 1;
 	static unsigned long long	atotsize, stotsize, maxshadowsz;
 	static time_t			reclast;
 	struct timespec			retrytimer = {0, RETRYMS/2*1000000};
 	int				retrycount = RETRYCNT;
-	int				asz, nsz, ssz;
-	char				abuf[16000], nbuf[16000];
+	int				asz, rv, ssz;
+	char				abuf[16000];
 	int				partsz, remsz;
 
 	/*
@@ -481,38 +517,73 @@ awaitprocterm(int nfd, int afd, int sfd, char *accountpath,
 	** source file as a trigger that a new accounting record
 	** has been written (does not work if the kernel itself
 	** writes to the file)
+	**
+	** when the NETLINK interface fails due to kernel bug 190711,
+	** we switch to polling mode:
+	**	wait for timer and verify if process accounting
+	**      records are available (repeatedly); ugly but the only
+	**	thing we can do if we can't use NETLINK
 	*/
-	nsz = recv(nfd, nbuf, sizeof nbuf, 0);
-
-	if (nsz == 0) 		// EOF?
+	if (netlinkactive)
 	{
-		syslog(LOG_ERR, "unexpected EOF on NETLINK\n");
-		return -1;
-	}
+		rv = netlink_recv(nfd, 0);
 
-	if (nsz == -1)		// failure?
-	{
-		switch (errno)
+		if (rv == 0) 		// EOF?
 		{
-		   // acceptable errors that might indicate that
-		   // processes have terminated
-		   case EINTR:
-		   case ENOMEM:
-		   case ENOBUFS:
-			break;
-
-		   default:
-			syslog(LOG_ERR, "unexpected error on NETLINK: %s\n",
-						strerror(errno));
+			syslog(LOG_ERR, "unexpected EOF on NETLINK\n");
+			perror("unexpected EOF on NETLINK\n");
 			return -1;
 		}
-	}
 
-	/*
- 	** get rid of all other waiting finished processes via netlink
-	** before handling the process accounting record(s)
-	*/
-	while (recv(nfd, nbuf, sizeof nbuf, MSG_DONTWAIT) > 0);
+		if (rv < 0)		// failure?
+		{
+			switch (-rv)
+			{
+		   	   // acceptable errors that might indicate that
+		   	   // processes have terminated
+		   	   case EINTR:
+		   	   case ENOMEM:
+		   	   case ENOBUFS:
+				break;
+	
+		   	   default:
+				syslog(LOG_ERR,
+					"unexpected error on NETLINK: %s\n",
+					strerror(-rv));
+				fprintf(stderr,
+					"unexpected error on NETLINK: %s\n",
+                                       	strerror(-rv));
+
+				if (-rv == EINVAL)
+				{
+				   	syslog(LOG_ERR,
+				   	"(see ATOP README about kernel bug 190711)\n");
+					fprintf(stderr,
+				   	"(see ATOP README about kernel bug 190711)\n");
+				}
+
+				syslog(LOG_ERR,
+					"switching to polling mode\n");
+				fprintf(stderr,
+					"switching to polling mode\n");
+
+				netlinkactive = 0;	// polling mode wanted
+
+				return 0;
+			}
+		}
+
+		/*
+ 		** get rid of all other waiting finished processes via netlink
+		** before handling the process accounting record(s)
+		*/
+		while ( netlink_recv(nfd, MSG_DONTWAIT) > 0 );
+	}
+	else	// POLLING MODE
+	{
+		sleep(POLLSEC);
+		retrycount = 1;
+	}
 
 	/*
  	** read new process accounting record(s)
@@ -528,15 +599,14 @@ awaitprocterm(int nfd, int afd, int sfd, char *accountpath,
 	switch (asz)
 	{
 	   case 0:		// EOF (no records available)?
-		if (time(0) > reclast + NORECINTERVAL)
+		if (time(0) > reclast + NORECINTERVAL && reclast)
 		{
 			syslog(LOG_WARNING, "reactivate process accounting\n");
 
 			if (truncate(accountpath, 0) != -1)
 			{
 				lseek(afd, 0, SEEK_SET);
-				atotsize = 0;
-				(void) acct(accountpath);
+				atotsize = swonpacct(afd, accountpath);
 			}
 
 			reclast = time(0);
@@ -764,6 +834,72 @@ pass2shadow(int sfd, char *sbuf, int ssz)
 	return ssz;
 }
 
+
+/*
+** switch on the process accounting mechanism
+**   first parameter:	file descriptor of open accounting file
+**   second parameter:	name of accounting file
+**   return value:	-1 in case of permanent failure,
+** 			otherwise number of bytes read from accounting file
+*/
+static int
+swonpacct(int afd, char *accountpath)
+{
+	int  n, acctokay = 0;
+	char abuf[4096];
+
+	/*
+	** due to kernel bug 190271 (process accounting sometimes
+	** does not work), we verify if process accounting really
+	** works after switching it on. If not, we keep retrying
+	** for a while.
+	*/
+	while (! acctokay)
+	{
+		int  maxcnt = 40;
+
+		/*
+		** switch on process accounting
+		*/
+		if ( acct(accountpath) == -1)
+		{
+			perror("cannot switch on process accounting");
+			return -1;
+		}
+
+		/*
+		** try if process accounting works by spawning a
+		** child process that immediately finishes (should
+		** result in a process accounting record)
+		*/
+		if ( fork() == 0 )	
+			exit(0);
+
+		(void) wait((int *)0);		// wait for child to finish
+
+		while ( (n = read(afd, abuf, sizeof abuf)) <= 0 && --maxcnt)
+			usleep(50000);
+
+		if (n > 0)			// process accounting works
+		{
+			acctokay = 1;
+		}
+		else
+		{
+			syslog(LOG_ERR,
+			   "Retrying to switch on process accounting\n");
+			syslog(LOG_ERR,
+			   "(see ATOP README about kernel bug 190271)\n");
+
+			acct((char *)0);	// switch off process accounting
+			sleep(PACCTSEC);	// wait a while before retrying
+		}
+	}
+
+	return n;
+}
+
+
 /*
 ** remove old shadow files not being in use any more
 **
