@@ -5,11 +5,11 @@
 ** the system on system-level as well as process-level.
 **
 ** ==========================================================================
-** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
-** E-mail:      gerlof@ATComputing.nl
+** Author:      Gerlof Langeveld
+** E-mail:      gerlof.langeveld@atoptool.nl
 ** Date:        January 2007
 ** --------------------------------------------------------------------------
-** Copyright (C) 2007 Gerlof Langeveld
+** Copyright (C) 2007-2010 Gerlof Langeveld
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -26,8 +26,11 @@
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ** --------------------------------------------------------------------------
 **
-** $Id: ifprop.c,v 1.4 2007/02/13 10:34:06 gerlof Exp $
+** $Id: ifprop.c,v 1.5 2010/04/23 12:19:35 gerlof Exp $
 ** $Log: ifprop.c,v $
+** Revision 1.5  2010/04/23 12:19:35  gerlof
+** Modified mail-address in header.
+**
 ** Revision 1.4  2007/02/13 10:34:06  gerlof
 ** Removal of external declarations.
 **
@@ -48,6 +51,11 @@ typedef __u32	u32;
 typedef __u16	u16;
 typedef __u8	u8;
 #include <linux/ethtool.h>
+#include <linux/wireless.h>
+
+#ifndef	SPEED_UNKNOWN
+#define	SPEED_UNKNOWN	-1
+#endif
 
 #include "atop.h"
 #include "ifprop.h"
@@ -76,6 +84,7 @@ getifprop(struct ifprop *ifp)
 		}
 	}
 
+	ifp->type	= '?';
 	ifp->speed	= 0;
 	ifp->fullduplex	= 0;
 
@@ -94,8 +103,9 @@ initifprop(void)
 	FILE 			*fp;
 	char 			*cp, linebuf[2048];
 	int			i=0, sockfd;
-	struct ifreq	 	ifreq;
 	struct ethtool_cmd 	ethcmd;
+	struct ifreq	 	ifreq;
+	struct iwreq	 	iwreq;
 
 	/*
 	** open /proc/net/dev to obtain all interface names and open
@@ -127,7 +137,7 @@ initifprop(void)
 		sscanf(linebuf, "%15s", ifprops[i].name);
 
 		/*
-		** determine properties of interface
+		** determine properties of ethernet interface
 		*/
 		memset(&ifreq,  0, sizeof ifreq);
 		memset(&ethcmd, 0, sizeof ethcmd);
@@ -139,37 +149,54 @@ initifprop(void)
 
 		ethcmd.cmd = ETHTOOL_GSET;
 
-		if ( ioctl(sockfd, SIOCETHTOOL, &ifreq) == -1) 
+		if ( ioctl(sockfd, SIOCETHTOOL, &ifreq) == 0) 
 		{
+			ifprops[i].type  = 'e';	// type ethernet
+			ifprops[i].speed = ethtool_cmd_speed(&ethcmd);
+
+			if (ifprops[i].speed == (u32)SPEED_UNKNOWN)
+				ifprops[i].speed = 0;
+
+			switch (ethcmd.duplex)
+			{
+		   	   case DUPLEX_FULL:
+				ifprops[i].fullduplex	= 1;
+				break;
+		   	   default:
+				ifprops[i].fullduplex	= 0;
+			}
+
 			if (++i >= MAXINTF-1)
 				break;
 			else
 				continue;
 		}
 
-		switch (ethcmd.speed)
+		/*
+		** determine properties of wireless interface
+		*/
+		memset(&iwreq,  0, sizeof iwreq);
+
+		strncpy(iwreq.ifr_ifrn.ifrn_name, ifprops[i].name,
+				sizeof iwreq.ifr_ifrn.ifrn_name-1);
+
+		if ( ioctl(sockfd, SIOCGIWRATE, &iwreq) == 0) 
 		{
-		   case SPEED_10:
-			ifprops[i].speed	= 10;
-			break;
-		   case SPEED_100:
-			ifprops[i].speed	= 100;
-			break;
-		   case SPEED_1000:
-			ifprops[i].speed	= 1000;
-			break;
-		   default:
-			ifprops[i].speed	= 0;
+			ifprops[i].type       = 'w';	// type wireless
+			ifprops[i].fullduplex = 0;
+
+			ifprops[i].speed =
+				(iwreq.u.bitrate.value + 500000) / 1000000;
+
+			if (++i >= MAXINTF-1)
+				break;
+			else
+				continue;
 		}
 
-		switch (ethcmd.duplex)
-		{
-		   case DUPLEX_FULL:
-			ifprops[i].fullduplex	= 1;
-			break;
-		   default:
-			ifprops[i].fullduplex	= 0;
-		}
+		ifprops[i].type       = '?';	// type unknown
+		ifprops[i].fullduplex = 0;
+		ifprops[i].speed      = 0;
 
 		if (++i >= MAXINTF-1)
 			break;
