@@ -59,7 +59,9 @@ struct tstat {
 		int	ctid;		/* OpenVZ container ID		*/
 		int	vpid;		/* OpenVZ virtual PID		*/
 
-		int	ifuture[5];     /* reserved                     */
+		int	wasinactive;	/* boolean: task inactive	*/
+
+		char	container[16];	/* Docker container id (12 pos)	*/
 	} gen;
 
 	/* CPU STATISTICS						*/
@@ -128,6 +130,22 @@ struct pinfo {
 	struct tstat	tstat;		/* per-process statistics        */
 };
 
+/*
+** structure to maintains all deviation info related to one sample
+*/
+struct devtstat {
+        struct tstat     *taskall;
+        struct tstat    **procall;
+        struct tstat    **procactive;
+
+	unsigned long	ntaskall;
+        unsigned long	ntaskactive;
+	unsigned long	nprocall;
+	unsigned long	nprocactive;
+
+        unsigned long   totrun, totslpi, totslpu, totzombie;
+};
+
 /*
 ** prototypes of process-database functions
 */
@@ -143,9 +161,9 @@ int		pdb_srchresidue(struct tstat *, struct pinfo **);
 */
 struct netpertask;
 
-int		deviattask(struct tstat *, int, struct tstat *, int, int,
-				struct tstat *, struct sstat *, unsigned int *,
-				int *, int *, int *, int *, int *);
+void		deviattask(struct tstat *, int,
+		           struct tstat *, int, 
+		           struct devtstat *, struct sstat *);
 
 int		photoproc(struct tstat *, int);
 unsigned int	countprocs(void);
