@@ -132,7 +132,7 @@ pdb_gettask(int pid, char isproc, time_t btime, struct pinfo **pinfopp)
 
 			*pinfopp = pp;
 
-			return(1);
+			return 1;
 		}
 
 		pp = pp->phnext;
@@ -141,7 +141,7 @@ pdb_gettask(int pid, char isproc, time_t btime, struct pinfo **pinfopp)
 	/*
 	** end of list; PID not found
 	*/
-	return(0);
+	return 0;
 }
 
 /*
@@ -184,7 +184,7 @@ pdb_deltask(int pid, char isproc)
 		*/
 		free(pp);
 
-		return(1);
+		return 1;
 	}
 
 	/*
@@ -214,14 +214,14 @@ pdb_deltask(int pid, char isproc)
 			*/
 			free(pp);
 
-			return(1);
+			return 1;
 		}
 
 		ppp	= pp;
 		pp	= pp->phnext;
 	}
 
-	return(0);	/* PID not found */
+	return 0;	/* PID not found */
 }
 
 /*
@@ -270,7 +270,7 @@ pdb_makeresidue(void)
 	/*
 	** all entries chained in doubly-linked RESIDUE-list
 	*/
-	return(1);
+	return 1;
 }
 
 /*
@@ -298,7 +298,7 @@ pdb_cleanresidue(void)
 		pdb_deltask(pid, isproc);
 	}
 
-	return(1);
+	return 1;
 }
 
 /*
@@ -340,7 +340,7 @@ pdb_srchresidue(struct tstat *tstatp, struct pinfo **pinfopp)
 			if (btimediff == 0)	/* gotcha !! */
 			{
 				*pinfopp = pr;
-				return(1);
+				return 1;
 			}
 
 			if ((btimediff== -1 || btimediff== 1) && prmin== NULL)
@@ -357,8 +357,8 @@ pdb_srchresidue(struct tstat *tstatp, struct pinfo **pinfopp)
 	if (prmin)
 	{
 		*pinfopp = prmin;
-		return(1);
+		return 1;
 	}
 
-	return(0);	/* even not almost */
+	return 0;	/* even not almost */
 }
