@@ -24,119 +24,8 @@
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ** --------------------------------------------------------------------------
-**
-** $Log: rawlog.c,v $
-** Revision 1.32  2010/11/26 06:06:35  gerlof
-** Cosmetic.
-**
-** Revision 1.31  2010/11/17 12:43:31  gerlof
-** The flag -r followed by exactly 8 'y' characters is not considered
-** as 8 days ago, but as a literal filename.
-**
-** Revision 1.30  2010/10/23 14:03:03  gerlof
-** Counters for total number of running and sleep threads (JC van Winkel).
-**
-** Revision 1.29  2010/04/23 13:55:52  gerlof
-** Get rid of all setuid-root privs before activating other program.
-**
-** Revision 1.28  2010/04/23 12:19:35  gerlof
-** Modified mail-address in header.
-**
-** Revision 1.27  2010/04/16 12:55:16  gerlof
-** Automatically start another version of atop if the logfile to
-** be read has not been created by the current version.
-**
-** Revision 1.26  2010/03/02 13:55:29  gerlof
-** Struct stat size did not fit in short any more (modified to int).
-**
-** Revision 1.25  2009/12/17 08:16:06  gerlof
-** Introduce branch-key to go to specific time in raw file.
-**
-** Revision 1.24  2009/11/27 15:26:29  gerlof
-** Added possibility to specify y[y..] als filename for -r flag
-** to access file of yesterday, day before yesterday, etc.
-**
-** Revision 1.23  2009/11/27 14:28:14  gerlof
-** Rollback a "transaction" when not all parts could be
-** written to the logfile (e.g. file system full) to avoid
-** a corrupted logfile.
-**
-** Revision 1.22  2008/01/07 10:18:05  gerlof
-** Implement possibility to make summaries.
-**
-** Revision 1.21  2007/08/16 12:01:25  gerlof
-** Add support for atopsar reporting.
-**
-** Revision 1.20  2007/03/20 13:02:25  gerlof
-** Introduction of variable supportflags.
-**
-** Revision 1.19  2007/03/20 11:11:57  gerlof
-** Verify success of malloc's.
-**
-** Revision 1.18  2007/03/20 07:25:59  gerlof
-** Avoid loop when incompatible raw file is read.
-** Verify return code of compress/uncompress functions.
-**
-** Revision 1.17  2007/02/23 07:34:00  gerlof
-** Changed unsigned short's into unsigned int's for process-counters
-** in raw record.
-**
-** Revision 1.16  2007/02/13 10:33:27  gerlof
-** Removal of external declarations.
-** Store pagesize and hertz in raw logfile.
-**
-** Revision 1.15  2006/01/30 09:12:34  gerlof
-** Minor bug-fix.
-**
-** Revision 1.14  2005/10/21 09:50:36  gerlof
-** Per-user accumulation of resource consumption.
-**
-** Revision 1.13  2004/12/14 15:06:23  gerlof
-** Implementation of patch-recognition for disk and network-statistics.
-**
-** Revision 1.12  2004/05/06 09:47:36  gerlof
-** Ported to kernel-version 2.6.
-**
-** Revision 1.11  2003/07/07 09:26:48  gerlof
-** Cleanup code (-Wall proof).
-**
-** Revision 1.10  2003/07/03 12:04:11  gerlof
-** Implemented subcommand `r' (reset).
-**
-** Revision 1.9  2003/06/27 12:32:48  gerlof
-** Removed rawlog compatibility.
-**
-** Revision 1.8  2003/02/06 14:08:33  gerlof
-** Cosmetic changes.
-**
-** Revision 1.7  2003/01/17 07:33:39  gerlof
-** Modified process statistics: add command-line.
-** Implement compatibility for old tstat-structure read from logfiles.
-**
-** Revision 1.6  2002/10/30 13:46:11  gerlof
-** Generate notification for statistics since boot.
-** Adapt interval from ushort to ulong (bug-solution);
-** this results in incompatible logfile but old logfiles can still be read.
-**
-** Revision 1.5  2002/10/24 12:22:25  gerlof
-** In case of writing a raw logfile:
-** If the logfile already exists, append new records to the existing file.
-** If the logfile does not yet exist, create it.
-**
-** Revision 1.4  2002/10/08 11:35:22  gerlof
-** Modified storage of raw filename.
-**
-** Revision 1.3  2002/10/03 10:41:51  gerlof
-** Avoid end-less loop when using end-time specification (flag -e).
-**
-** Revision 1.2  2002/09/17 13:17:01  gerlof
-** Allow key 'T' to be pressed to view previous sample in raw file.
-**
-**
 */
 
-static const char rcsid[] = "$Id: rawlog.c,v 1.32 2010/11/26 06:06:35 gerlof Exp $";
-
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/mman.h>
@@ -239,9 +128,7 @@ static void	try_other_version(int, int);
 */
 char
 rawwrite(time_t curtime, int numsecs, 
-	 struct sstat *ss, struct tstat *ts, struct tstat **proclist,
-	 int ndeviat, int ntask, int nactproc,
-         int totproc, int totrun, int totslpi, int totslpu, int totzomb, 
+         struct devtstat *devtstat, struct sstat *sstat,
          int nexit, unsigned int noverflow, char flag)
 {
 	static int		rawfd = -1;
@@ -251,7 +138,8 @@ rawwrite(time_t curtime, int numsecs,
 
 	Byte			scompbuf[sizeof(struct sstat)], *pcompbuf;
 	unsigned long		scomplen = sizeof scompbuf;
-	unsigned long		pcomplen = sizeof(struct tstat) * ndeviat;
+	unsigned long		pcomplen = sizeof(struct tstat) *
+						devtstat->ntaskall;
 
 	/*
 	** first call:
@@ -271,7 +159,7 @@ rawwrite(time_t curtime, int numsecs,
 	** compress system- and process-level statistics
 	*/
 	rv = compress(scompbuf, &scomplen,
-				(Byte *)ss, (unsigned long)sizeof *ss);
+				(Byte *)sstat, (unsigned long)sizeof *sstat);
 
 	testcompval(rv, "compress");
 
@@ -279,7 +167,8 @@ rawwrite(time_t curtime, int numsecs,
 
 	ptrverify(pcompbuf, "Malloc failed for compression buffer\n");
 
-	rv = compress(pcompbuf, &pcomplen, (Byte *)ts, (unsigned long)pcomplen);
+	rv = compress(pcompbuf, &pcomplen, (Byte *)devtstat->taskall,
+						(unsigned long)pcomplen);
 
 	testcompval(rv, "compress");
 
@@ -291,16 +180,16 @@ rawwrite(time_t curtime, int numsecs,
 	rr.curtime	= curtime;
 	rr.interval	= numsecs;
 	rr.flags	= 0;
-	rr.ndeviat	= ndeviat;
-	rr.nactproc	= nactproc;
-	rr.ntask	= ntask;
+	rr.ndeviat	= devtstat->ntaskall;
+	rr.nactproc	= devtstat->nprocactive;
+	rr.ntask	= devtstat->ntaskall;
 	rr.nexit	= nexit;
 	rr.noverflow	= noverflow;
-	rr.totproc	= totproc;
-	rr.totrun	= totrun;
-	rr.totslpi	= totslpi;
-	rr.totslpu	= totslpu;
-	rr.totzomb	= totzomb;
+	rr.totproc	= devtstat->nprocall;
+	rr.totrun	= devtstat->totrun;
+	rr.totslpi	= devtstat->totslpi;
+	rr.totslpu	= devtstat->totslpu;
+	rr.totzomb	= devtstat->totzombie;
 	rr.scomplen	= scomplen;
 	rr.pcomplen	= pcomplen;
 
@@ -319,6 +208,9 @@ rawwrite(time_t curtime, int numsecs,
 	if (supportflags & NETATOPD)
 		rr.flags |= RRNETATOPD;
 
+	if (supportflags & DOCKSTAT)
+		rr.flags |= RRDOCKSTAT;
+
 	if ( write(rawfd, &rr, sizeof rr) == -1)
 	{
 		fprintf(stderr, "%s - ", rawname);
@@ -403,7 +295,7 @@ rawwopen()
 		if ( rh.sstatlen	!= sizeof(struct sstat)		||
 		     rh.tstatlen	!= sizeof(struct tstat)		||
 	    	     rh.rawheadlen	!= sizeof(struct rawheader)	||
-		     rh.rawreclen	!= sizeof(struct rawrecord)       )
+		     rh.rawreclen	!= sizeof(struct rawrecord)	  )
 		{
 			fprintf(stderr,
 				"existing file %s has incompatible header\n",
@@ -478,9 +370,8 @@ rawread(void)
 	char			*py;
 	struct rawheader	rh;
 	struct rawrecord	rr;
-	struct sstat		devsstat;
-	struct tstat		*devtstat;
-	struct tstat		**devpstat;
+	struct sstat		sstat;
+	static struct devtstat	devtstat;
 
 	struct stat		filestat;
 
@@ -707,6 +598,7 @@ rawread(void)
 		while ( getrawrec(rawfd, &rr, rh.rawreclen) == rh.rawreclen)
 		{
 			unsigned int	secsinday = daysecs(rr.curtime);
+			unsigned int	k, l;
 
 			/*
 			** store the offset of the raw record in the offset list
@@ -750,29 +642,66 @@ rawread(void)
 			** allocate space, read compressed system-level
 			** statistics and decompress
 			*/
-			if ( !getrawsstat(rawfd, &devsstat, rr.scomplen) )
+			if ( !getrawsstat(rawfd, &sstat, rr.scomplen) )
 				cleanstop(7);
 
 			/*
 			** allocate space, read compressed process-level
 			** statistics and decompress
 			*/
-			devtstat = malloc(sizeof(struct tstat) * rr.ndeviat);
+			devtstat.taskall    = malloc(sizeof(struct tstat  )
+								* rr.ndeviat);
 
-			ptrverify(devtstat,
+			if (rr.totproc < rr.nactproc)	// compat old raw files
+				devtstat.procall = malloc(sizeof(struct tstat *)
+								* rr.nactproc);
+			else
+				devtstat.procall = malloc(sizeof(struct tstat *)
+								* rr.totproc);
+
+			devtstat.procactive = malloc(sizeof(struct tstat *) * rr.nactproc);
+
+			ptrverify(devtstat.taskall,
 			          "Malloc failed for %d stored tasks\n",
 			          rr.ndeviat);
 
-			devpstat = malloc(sizeof(struct tstat *) * rr.nactproc);
+			ptrverify(devtstat.procall,
+			          "Malloc failed for total %d processes\n",
+			          rr.totproc);
 
-			ptrverify(devpstat,
-			          "Malloc failed for %d stored processes\n",
+			ptrverify(devtstat.procactive,
+			          "Malloc failed for %d active processes\n",
 			          rr.nactproc);
 
-			if ( !getrawtstat(rawfd, devtstat,
+			if ( !getrawtstat(rawfd, devtstat.taskall,
 						rr.pcomplen, rr.ndeviat) )
 				cleanstop(7);
 
+
+			for (i=j=k=l=0; i < rr.ndeviat; i++)
+			{
+				if ( (devtstat.taskall+i)->gen.isproc)
+				{
+					devtstat.procall[j++] = devtstat.taskall+i;
+
+					if (! (devtstat.taskall+i)->gen.wasinactive)
+						devtstat.procactive[k++] = devtstat.taskall+i;
+				}
+
+				if (! (devtstat.taskall+i)->gen.wasinactive)
+					l++;
+			}
+
+			devtstat.ntaskall	= i;
+			devtstat.nprocall	= j;
+			devtstat.nprocactive	= k;
+			devtstat.ntaskactive	= l;
+
+ 			devtstat.totrun		= rr.totrun;
+ 			devtstat.totslpi	= rr.totslpi;
+ 			devtstat.totslpu	= rr.totslpu;
+ 			devtstat.totzombie	= rr.totzomb;
+
 			/*
 			** activate the installed print-function to visualize
 			** the system- and process-level statistics
@@ -785,7 +714,7 @@ rawread(void)
 					supportflags |=  ACCTACTIVE;
 				else
 					supportflags &= ~ACCTACTIVE;
-			
+
 				if (rr.flags & RRIOSTAT)
 					supportflags |=  IOSTAT;
 				else
@@ -802,6 +731,11 @@ rawread(void)
 			else
 				supportflags &= ~NETATOPD;
 
+			if (rr.flags & RRDOCKSTAT)
+				supportflags |=  DOCKSTAT;
+			else
+				supportflags &= ~DOCKSTAT;
+
 			flags = rr.flags & RRBOOT;
 
 			(void) fstat(rawfd, &filestat);
@@ -810,21 +744,13 @@ rawread(void)
 			     lseek(rawfd, (off_t)0, SEEK_CUR) <= rh.rawreclen)
 				flags |= RRLAST;
 
-			for (i=j=0; i < rr.ndeviat; i++)
-			{
-				if ( (devtstat+i)->gen.isproc)
-					devpstat[j++] = devtstat+i;
-			}
-
 			lastcmd = (vis.show_samp)(rr.curtime, rr.interval,
-			             &devsstat, devtstat, devpstat,
-		 	             rr.ndeviat,  rr.ntask, rr.nactproc,
-			             rr.totproc, rr.totrun, rr.totslpi,
-			             rr.totslpu, rr.totzomb, rr.nexit,
-			             rr.noverflow, flags);
+			             &devtstat, &sstat,
+			             rr.nexit, rr.noverflow, flags);
 
-			free(devtstat);
-			free(devpstat);
+			free(devtstat.taskall);
+			free(devtstat.procall);
+			free(devtstat.procactive);
 	
 			switch (lastcmd)
 			{
