@@ -290,7 +290,7 @@ static int	paused;     	/* boolean: currently in pause-mode     */
 static int	fixedhead;	/* boolean: fixate header-lines         */
 static int	sysnosort;	/* boolean: suppress sort of resources  */
 static int	avgval;		/* boolean: average values i.s.o. total */
-static int	supexits;	/* boolean: suppress exited processes   */
+static int	suppressexit;	/* boolean: suppress exited processes   */
 
 static char	showtype  = MPROCGEN;
 static char	showorder = MSORTCPU;
@@ -309,7 +309,10 @@ static short	colorcrit   = COLOR_RED;
 static short	colorthread = COLOR_YELLOW;
 
 static int	cumusers(struct tstat **, struct tstat *, int);
-static int	cumprocs(struct tstat **, struct tstat *, int);
+static int	cumprogs(struct tstat **, struct tstat *, int);
+static int	cumconts(struct tstat **, struct tstat *, int);
+static void	accumulate(struct tstat *, struct tstat *);
+
 static int	procsuppress(struct tstat *, struct pselection *);
 static void	limitedlines(void);
 static long	getnumval(char *, long, int);
@@ -335,14 +338,14 @@ int	startoffset;
 */
 char
 generic_samp(time_t curtime, int nsecs,
-           struct sstat *sstat, struct tstat *tstat, struct tstat **proclist,
-           int ndeviat, int ntask, int nactproc,
-           int totproc, int totrun, int totslpi, int totslpu, int totzomb,
+           struct devtstat *devtstat, struct sstat *sstat, 
            int nexit, unsigned int noverflow, char flag)
 {
 	static int	callnr = 0;
+	char		*p;
 
-	register int	i, curline, statline;
+
+	register int	i, curline, statline, nproc;
 	int		firstproc = 0, plistsz, alistsz, killpid, killsig;
 	int		lastchar;
 	char		format1[16], format2[16], hhmm[16];
@@ -354,47 +357,54 @@ generic_samp(time_t curtime, int nsecs,
 
 	/*
 	** curlist points to the active list of tstat-pointers that
-	** should be displayed; nlist indicates the number of entries in
+	** should be displayed; ncurlist indicates the number of entries in
 	** this list
 	*/
 	struct tstat	**curlist;
-	int		nlist;
+	int		ncurlist;
 
 	/*
 	** tXcumlist is a list of tstat-structs holding one entry
 	** per accumulated (per user or per program) group of processes
 	**
 	** Xcumlist contains the pointers to all structs in tXcumlist
-	**
 	** 
-	** these lists will only be allocated 'lazy' whenever accumulation
-	** is requested
+	** these lists will only be allocated 'lazy'
+	** only when accumulation is requested
 	*/
-	struct tstat	*tpcumlist = 0;
+	struct tstat	*tpcumlist = 0;		// per program accumulation
 	struct tstat	**pcumlist = 0;
 	int		npcum      = 0;
 	char		plastorder = 0;
 
-	struct tstat	*tucumlist = 0;
+	struct tstat	*tucumlist = 0;		// per user accumulation
 	struct tstat	**ucumlist = 0;
 	int		nucum      = 0;
 	char		ulastorder = 0;
 
+	struct tstat	*tccumlist = 0;		// per container accumulation
+	struct tstat	**ccumlist = 0;
+	int		nccum      = 0;
+	char		clastorder = 0;
+
 	/*
 	** tsklist contains the pointers to all structs in tstat
-	** (number of entries: ndeviat), sorted on process with the
-	** related threads immediately following the process
+	** sorted on process with the related threads immediately
+	** following the process
 	**
 	** this list will be allocated 'lazy'
 	*/
 	struct tstat	**tsklist  = 0;
+	int		ntsk       = 0;
 	char		tlastorder = 0;
 	char		zipagain   = 0;
+	char		tdeviate   = 0;
 
 	/*
 	** sellist contains the pointers to the structs in tstat
 	** that are currently selected on basis of a particular
-	** username, program name (both regexp's) or suppressed exited procs
+	** username (regexp), program name (regexp), container name
+	** or suppressed exited procs
 	**
 	** this list will be allocated 'lazy'
 	*/
@@ -416,7 +426,7 @@ generic_samp(time_t curtime, int nsecs,
 	** compute the total capacity of this system for the 
 	** four main resources
 	*/
-	totalcap(&syscap, sstat, proclist, nactproc);
+	totalcap(&syscap, sstat, devtstat->procactive, devtstat->nprocactive);
 
 	/*
 	** sort per-cpu       		statistics on busy percentage
@@ -486,11 +496,12 @@ generic_samp(time_t curtime, int nsecs,
 
                 int seclen	= val2elapstr(nsecs, buf);
                 int lenavail 	= (screen ? COLS : linelen) -
-						48 - seclen - utsnodenamelen;
+						49 - seclen - utsnodenamelen;
                 int len1	= lenavail / 3;
                 int len2	= lenavail - len1 - len1; 
 
-		printg("ATOP - %s%*s%s  %s%*s%c%c%c%c%c%c%c%c%c%c%c%c%c%*s%s elapsed", 
+		printg("ATOP - %s%*s%s  %s%*s%c%c%c%c%c%c%c%c%c%c%c%c%c%c%*s%s"
+		       " elapsed", 
 			utsname.nodename, len1, "", 
 			format1, format2, len1, "",
 			threadview                    ? MTHREAD    : '-',
@@ -500,9 +511,10 @@ generic_samp(time_t curtime, int nsecs,
 			usecolors  		      ? '-'        : MCOLORS,
 			avgval     		      ? MAVGVAL    : '-',
 			calcpss     		      ? MCALCPSS   : '-',
-			supexits     		      ? MSUPEXITS  : '-',
+			suppressexit 		      ? MSUPEXITS  : '-',
 			procsel.userid[0] != USERSTUB ? MSELUSER   : '-',
 			procsel.prognamesz	      ? MSELPROC   : '-',
+			procsel.container[0]	      ? MSELCONT   : '-',
 			procsel.pid[0] != 0	      ? MSELPID    : '-',
 			procsel.argnamesz	      ? MSELARG    : '-',
 			syssel.lvmnamesz +
@@ -518,15 +530,14 @@ generic_samp(time_t curtime, int nsecs,
 		/*
 		** print cumulative system- and user-time for all processes
 		*/
-		pricumproc(sstat, proclist, nactproc, ntask,
-			totproc, totrun, totslpi, totslpu, totzomb,
-			nexit, noverflow, avgval, nsecs);
+		pricumproc(sstat, devtstat, nexit, noverflow, avgval, nsecs);
 
 		if (noverflow)
 		{
 			snprintf(statbuf, sizeof statbuf, 
 			         "Only %d exited processes handled "
 			         "-- %u skipped!", nexit, noverflow);
+
 			statmsg = statbuf;
 		}
 
@@ -661,29 +672,39 @@ generic_samp(time_t curtime, int nsecs,
 				ulastorder = 0;
 			}
 
+			if (deviatonly)
+				nproc = devtstat->nprocactive;
+			else
+				nproc = devtstat->nprocall;
+
 			/*
 			** allocate space for new (temporary) list with
 			** one entry per user (list has worst-case size)
 			*/
-			tucumlist = calloc(sizeof(struct tstat),    nactproc);
-			ucumlist  = malloc(sizeof(struct tstat *) * nactproc);
+			tucumlist = calloc(sizeof(struct tstat),    nproc);
+			ucumlist  = malloc(sizeof(struct tstat *) * nproc);
 
 			ptrverify(tucumlist,
-			        "Malloc failed for %d ucum procs\n", nactproc);
+			        "Malloc failed for %d ucum procs\n", nproc);
 			ptrverify(ucumlist,
-			        "Malloc failed for %d ucum ptrs\n",  nactproc);
+			        "Malloc failed for %d ucum ptrs\n",  nproc);
 
-			for (i=0; i < nactproc; i++)	/* fill pointers */
+			for (i=0; i < nproc; i++)
+			{
+				/* fill pointers */
 				ucumlist[i] = tucumlist+i;
+			}
 
-			nucum = cumusers(proclist, tucumlist, nactproc);
+			nucum = cumusers(deviatonly ?
+						devtstat->procactive :
+						devtstat->procall,
+						tucumlist, nproc);
 
 			curlist   = ucumlist;
-			nlist     = nucum;
+			ncurlist  = nucum;
 			lastsortp = &ulastorder;
 			break;
 
-
 		   case MCUMPROC:
 			threadallowed = 0;
 
@@ -694,70 +715,134 @@ generic_samp(time_t curtime, int nsecs,
 				plastorder = 0;
 			}
 
+			if (deviatonly)
+				nproc = devtstat->nprocactive;
+			else
+				nproc = devtstat->nprocall;
+
 			/*
 			** allocate space for new (temporary) list with
 			** one entry per program (list has worst-case size)
 			*/
-			tpcumlist = calloc(sizeof(struct tstat),    nactproc);
-			pcumlist  = malloc(sizeof(struct tstat *) * nactproc);
+			tpcumlist = calloc(sizeof(struct tstat),    nproc);
+			pcumlist  = malloc(sizeof(struct tstat *) * nproc);
 
 			ptrverify(tpcumlist,
-			        "Malloc failed for %d pcum procs\n", nactproc);
+			        "Malloc failed for %d pcum procs\n", nproc);
 			ptrverify(pcumlist,
-			        "Malloc failed for %d pcum ptrs\n",  nactproc);
+			        "Malloc failed for %d pcum ptrs\n",  nproc);
 
-			for (i=0; i < nactproc; i++)	/* fill pointers */
+			for (i=0; i < nproc; i++)
+			{	
+				/* fill pointers */
 				pcumlist[i] = tpcumlist+i;
+			}
 
-			npcum = cumprocs(proclist, tpcumlist, nactproc);
+			npcum = cumprogs(deviatonly ?
+						devtstat->procactive :
+						devtstat->procall,
+						tpcumlist, nproc);
 
 			curlist   = pcumlist;
-			nlist     = npcum;
+			ncurlist  = npcum;
 			lastsortp = &plastorder;
 			break;
 
+		   case MCUMCONT:
+			threadallowed = 0;
+
+			if (ccumlist)	/* previous list still available? */
+			{
+                                free(ccumlist);
+                                free(tccumlist);
+				clastorder = 0;
+			}
+
+			if (deviatonly)
+				nproc = devtstat->nprocactive;
+			else
+				nproc = devtstat->nprocall;
+
+			/*
+			** allocate space for new (temporary) list with
+			** one entry per user (list has worst-case size)
+			*/
+			tccumlist = calloc(sizeof(struct tstat),    nproc);
+			ccumlist  = malloc(sizeof(struct tstat *) * nproc);
+
+			ptrverify(tccumlist,
+			        "Malloc failed for %d ccum procs\n", nproc);
+			ptrverify(ccumlist,
+			        "Malloc failed for %d ccum ptrs\n",  nproc);
+
+			for (i=0; i < nproc; i++)
+			{
+				/* fill pointers */
+				ccumlist[i] = tccumlist+i;
+			}
+
+			nccum = cumconts(deviatonly ?
+						devtstat->procactive :
+						devtstat->procall,
+						tccumlist, nproc);
+
+			curlist   = ccumlist;
+			ncurlist  = nccum;
+			lastsortp = &clastorder;
+			break;
+
 		   default:
 			threadallowed = 1;
 
+			if (deviatonly && showtype  != MPROCMEM &&
+			                  showorder != MSORTMEM   )
+			{
+				curlist   = devtstat->procactive;
+				ncurlist  = devtstat->nprocactive;
+			}
+			else
+			{
+				curlist   = devtstat->procall;
+				ncurlist  = devtstat->nprocall;
+			}
+
+			lastsortp = &tlastorder;
+
 			if ( procsel.userid[0] == USERSTUB &&
 			    !procsel.prognamesz            &&
+			    !procsel.container[0]          &&
 			    !procsel.argnamesz             &&
 			    !procsel.pid[0]                &&
-			    !supexits                        )
-			{	/* no selection wanted */
-				curlist   = proclist;
-				nlist     = nactproc;
-				lastsortp = &tlastorder;
+			    !suppressexit                    )
+				/* no selection wanted */
 				break;
-			}
 
 			/*
 			** selection specified for tasks:
 			** create new (worst case) pointer list if needed
 			*/
-			if (!sellist)
-			{
-				sellist = malloc(sizeof(struct tstat *)
-								* nactproc);
+			if (sellist)	// remove previous list if needed
+				free(sellist);
 
-				ptrverify(sellist,
-				          "Malloc failed for %d select ptrs\n",
-				          nactproc);
-			}
+			sellist = malloc(sizeof(struct tstat *) * ncurlist);
 
-			for (i=nsel=0; i < nactproc; i++)
+			ptrverify(sellist,
+			       "Malloc failed for %d select ptrs\n", ncurlist);
+
+			for (i=nsel=0; i < ncurlist; i++)
 			{
-				if (procsuppress(*(proclist+i), &procsel))
+				if (procsuppress(*(curlist+i), &procsel))
 					continue;
 
-				if ((proclist[i])->gen.state == 'E' && supexits)
+				if (curlist[i]->gen.state == 'E' &&
+				    suppressexit                   )
 					continue;
 
-				sellist[nsel++] = proclist[i]; 
+				sellist[nsel++] = curlist[i]; 
 			}
 
 			curlist    = sellist;
-			nlist      = nsel;
+			ncurlist   = nsel;
 			tlastorder = 0; /* new sort and zip normal view */
 			slastorder = 0;	/* new sort and zip now         */
 			lastsortp  = &slastorder;
@@ -779,18 +864,19 @@ generic_samp(time_t curtime, int nsecs,
 			plistsz = LINES-curline-2;
 		else
 			if (threadview && threadallowed)
-				plistsz = ndeviat;
+				plistsz = devtstat->ntaskactive;
 			else
-				plistsz = nlist;
+				plistsz = ncurlist;
 
-		if (nlist > 0 && plistsz > 0)
+
+		if (ncurlist > 0 && plistsz > 0)
 		{
 			/*
  			** if sorting order is changed, sort again
  			*/
 			if (*lastsortp != curorder)
 			{
-				qsort(curlist, nlist,
+				qsort(curlist, ncurlist,
 				        sizeof(struct tstat *),
 				        procsort[(int)curorder&0x1f]);
 
@@ -801,46 +887,77 @@ generic_samp(time_t curtime, int nsecs,
 
 			if (threadview && threadallowed)
 			{
-				int j = ndeviat, t;
+				int ntotal, j, t;
+
+				if (deviatonly && showtype  != MPROCMEM &&
+			      	                  showorder != MSORTMEM   )
+					ntotal = devtstat->ntaskactive;
+				else
+					ntotal = devtstat->ntaskall;
 
 				/*
-				** allocate new pointer list to be able to zip
-				** process list with references to the threads
+  				** check if existing pointer list still usable
+				** if not, allocate new pointer list to be able
+				** to zip process list with references to threads
 				*/
-				if (!tsklist)
+				if (!tsklist || ntsk != ntotal ||
+							tdeviate != deviatonly)
 				{
+					if (tsklist)
+						free(tsklist);	// remove current
+
 					tsklist = malloc(sizeof(struct tstat *)
-								    * ndeviat);
+								    * ntotal);
 
 					ptrverify(tsklist,
 				             "Malloc failed for %d taskptrs\n",
-				             ndeviat);
+				             ntotal);
+
+					ntsk     = ntotal;
+					tdeviate = deviatonly;
 
 					zipagain = 1;
 				}
+				else
+					j = ntotal;
 
 				if (zipagain)
 				{
-					for (i=j=0; i < nlist; i++)
-					{
-						tsklist[j++] = curlist[i];
+					struct tstat *tall = devtstat->taskall;
+					struct tstat *pcur;
 
-						for (t = curlist[i] - tstat+1;
-					     	     t < ndeviat &&
-						     (curlist[i])->gen.tgid &&
-					             (tstat+t)->gen.tgid ==
-					                (curlist[i])->gen.tgid;
-						     t++)
+					for (i=j=0; i < ncurlist; i++)
+					{
+					    pcur = curlist[i];
+
+					    tsklist[j++] = pcur;
+
+					    for (t = pcur - tall + 1;
+					         t < devtstat->ntaskall &&
+						 pcur->gen.tgid		&&
+					         pcur->gen.tgid == 
+					            (tall+t)->gen.tgid;
+					         t++)
+					    {
+						if (deviatonly &&
+							showtype  != MPROCMEM &&
+						        showorder != MSORTMEM   )
 						{
-							tsklist[j++] = tstat+t;
-						}
+						  if (!(tall+t)->gen.wasinactive)
+						  {
+							tsklist[j++] = tall+t;
+						  }
+ 						}
+						else
+							tsklist[j++] = tall+t;
+					    }
 					}
 
 					zipagain = 0;
 				}
 
-				curlist = tsklist;
-				nlist   = j;
+				curlist  = tsklist;
+				ncurlist = j;
 			}
 
 			/*
@@ -848,12 +965,13 @@ generic_samp(time_t curtime, int nsecs,
 			** first determine the column-header for the current
 			** sorting order of processes
 			*/
-			if (screen) {
+			if (screen)
+			{
 				attron(A_REVERSE);
                                 move(curline+1, 0);
                         }
 
-			priphead(firstproc/plistsz+1, (nlist-1)/plistsz+1,
+			priphead(firstproc/plistsz+1, (ncurlist-1)/plistsz+1,
 			       		&showtype, &curorder,
 					showorder == MSORTAUTO ? 1 : 0);
 
@@ -866,12 +984,12 @@ generic_samp(time_t curtime, int nsecs,
 			/*
 			** print the list
 			*/
-			priproc(curlist, firstproc, nlist, curline+2,
-			        firstproc/plistsz+1, (nlist-1)/plistsz+1,
+			priproc(curlist, firstproc, ncurlist, curline+2,
+			        firstproc/plistsz+1, (ncurlist-1)/plistsz+1,
 			        showtype, curorder, &syscap, nsecs, avgval);
 		}
 
-		alistsz = nlist;	/* preserve size of active list */
+		alistsz = ncurlist;	/* preserve size of active list */
 
 		/*
 		** in case of writing to a terminal, the user can also enter
@@ -909,6 +1027,8 @@ generic_samp(time_t curtime, int nsecs,
 				if (pcumlist)  free(pcumlist);
 				if (tucumlist) free(tucumlist);
 				if (ucumlist)  free(ucumlist);
+				if (tccumlist) free(tccumlist);
+				if (ccumlist)  free(ccumlist);
 				if (tsklist)   free(tsklist);
 				if (sellist)   free(sellist);
 
@@ -936,6 +1056,8 @@ generic_samp(time_t curtime, int nsecs,
 				if (pcumlist)  free(pcumlist);
 				if (tucumlist) free(tucumlist);
 				if (ucumlist)  free(ucumlist);
+				if (tccumlist) free(tccumlist);
+				if (ccumlist)  free(ccumlist);
 				if (tsklist)   free(tsklist);
 				if (sellist)   free(sellist);
 
@@ -960,6 +1082,8 @@ generic_samp(time_t curtime, int nsecs,
 				if (pcumlist)  free(pcumlist);
 				if (tucumlist) free(tucumlist);
 				if (ucumlist)  free(ucumlist);
+				if (tccumlist) free(tccumlist);
+				if (ccumlist)  free(ccumlist);
 				if (tsklist)   free(tsklist);
 				if (sellist)   free(sellist);
 
@@ -1003,6 +1127,8 @@ generic_samp(time_t curtime, int nsecs,
 				if (pcumlist)  free(pcumlist);
 				if (tucumlist) free(tucumlist);
 				if (ucumlist)  free(ucumlist);
+				if (tccumlist) free(tccumlist);
+				if (ccumlist)  free(ccumlist);
 				if (tsklist)   free(tsklist);
 				if (sellist)   free(sellist);
 
@@ -1191,6 +1317,17 @@ generic_samp(time_t curtime, int nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** accumulated resource consumption per container
+			   */
+			   case MCUMCONT:
+				statmsg = "Consumption per container; use 'a' to "
+				          "toggle between all/active processes";
+
+				showtype  = MCUMCONT;
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** help wanted?
 			   */
@@ -1421,6 +1558,70 @@ generic_samp(time_t curtime, int nsecs,
 				firstproc = 0;
 				break;
 
+			   /*
+			   ** focus on specific container id
+			   */
+			   case MSELCONT:
+				alarm(0);	/* stop the clock */
+				echo();
+
+				move(statline, 0);
+				clrtoeol();
+				printw("Containerid 12 postitions "
+ 				       "(enter=all, "
+				       "'host'=host processes): ");
+
+				procsel.container[0]  = '\0';
+				scanw("%15s", procsel.container);
+				procsel.container[12] = '\0';
+
+				switch (strlen(procsel.container))
+				{
+                                   case 0:
+					break;	// enter key pressed
+
+				   case 4:	// host?
+					if (strcmp(procsel.container, "host"))
+					{
+						statmsg="Invalid containerid!";
+						beep();
+						procsel.container[0] = '\0';
+					}
+					else
+					{
+						procsel.container[0] = 'H';
+						procsel.container[1] = '\0';
+					}
+					break;
+
+				   case 12:	// container id
+					(void)strtol(procsel.container, &p, 16);
+
+					if (*p)
+					{
+						statmsg ="Containerid not hex!";
+						beep();
+						procsel.container[0] = '\0';
+					}
+					break;
+
+				   default:
+					statmsg = "Invalid containerid!";
+					beep();
+
+					procsel.container[0] = '\0';
+				}
+
+				noecho();
+
+				move(statline, 0);
+
+				if (interval && !paused && !rawreadflag)
+					alarm(3);  /* set short timer */
+
+				firstproc = 0;
+				break;
+
 			   /*
 			   ** focus on specific PIDs
 			   */
@@ -1641,13 +1842,6 @@ generic_samp(time_t curtime, int nsecs,
 			   ** all processes
 			   */
 			   case MALLPROC:
-				if (rawreadflag)
-				{
-					statmsg = "Process list from raw file "
-					          "will be shown anyhow!";
-					break;
-				}
-
 				if (deviatonly)
 				{
 					deviatonly=0;
@@ -1661,10 +1855,8 @@ generic_samp(time_t curtime, int nsecs,
 					          "will be shown/accumulated...";
 				}
 
-				if (interval && !paused && !rawreadflag)
-					alarm(3);  /* set short timer */
-
-				firstproc = 0;
+				tlastorder = 0;
+				firstproc  = 0;
 				break;
 
 			   /*
@@ -1758,19 +1950,19 @@ generic_samp(time_t curtime, int nsecs,
 			   ** suppression of exited processes in output
 			   */
 			   case MSUPEXITS:
-				if (supexits)
+				if (suppressexit)
 				{
-					supexits    = 0;
-					statmsg    = "Exited processes will "
-					             "be shown/accumulated";
-					firstproc  = 0;
+					suppressexit = 0;
+					statmsg      = "Exited processes will "
+					               "be shown/accumulated";
+					firstproc    = 0;
 				}
 				else
 				{
-					supexits    = 1;
-					statmsg    = "Exited processes will "
+					suppressexit = 1;
+					statmsg      = "Exited processes will "
 					             "not be shown/accumulated";
-					firstproc  = 0;
+					firstproc    = 0;
 				}
 				break;
 
@@ -1866,6 +2058,8 @@ generic_samp(time_t curtime, int nsecs,
 				if (pcumlist)  free(pcumlist);
 				if (tucumlist) free(tucumlist);
 				if (ucumlist)  free(ucumlist);
+				if (tccumlist) free(tccumlist);
+				if (ccumlist)  free(ccumlist);
 				if (tsklist)   free(tsklist);
 				if (sellist)   free(sellist);
 
@@ -1905,7 +2099,6 @@ generic_samp(time_t curtime, int nsecs,
 			   */
 			   case KEY_DOWN:
 				if (firstproc < alistsz-1)
-				  //  alistsz-firstproc >= plistsz)
 					firstproc += 1;
 				break;
 
@@ -1964,6 +2157,8 @@ generic_samp(time_t curtime, int nsecs,
 			if (pcumlist)  free(pcumlist);
 			if (tucumlist) free(tucumlist);
 			if (ucumlist)  free(ucumlist);
+			if (tccumlist) free(tccumlist);
+			if (ccumlist)  free(ccumlist);
 			if (tsklist)   free(tsklist);
 			if (sellist)   free(sellist);
 
@@ -1979,7 +2174,6 @@ static int
 cumusers(struct tstat **curprocs, struct tstat *curusers, int numprocs)
 {
 	register int	i, numusers;
-	count_t		nett_wsz;
 
 	/*
 	** sort list of active processes in order of uid (increasing)
@@ -1994,7 +2188,7 @@ cumusers(struct tstat **curprocs, struct tstat *curusers, int numprocs)
 		if (procsuppress(*curprocs, &procsel))
 			continue;
 
-		if ((*curprocs)->gen.state == 'E' && supexits)
+		if ((*curprocs)->gen.state == 'E' && suppressexit)
 			continue;
  
 		if ( curusers->gen.ruid != (*curprocs)->gen.ruid )
@@ -2007,46 +2201,7 @@ cumusers(struct tstat **curprocs, struct tstat *curusers, int numprocs)
 			curusers->gen.ruid = (*curprocs)->gen.ruid;
 		}
 
-		curusers->gen.pid++;		/* misuse as counter */
-
-		curusers->gen.isproc  = 1;
-		curusers->gen.nthr   += (*curprocs)->gen.nthr;
-		curusers->cpu.utime  += (*curprocs)->cpu.utime;
-		curusers->cpu.stime  += (*curprocs)->cpu.stime;
-
- 		if ((*curprocs)->dsk.wsz > (*curprocs)->dsk.cwsz)
-                	nett_wsz = (*curprocs)->dsk.wsz -(*curprocs)->dsk.cwsz;
-		else
-			nett_wsz = 0;
-
-		curusers->dsk.rio    += (*curprocs)->dsk.rsz;
-		curusers->dsk.wio    += nett_wsz;
-
-		curusers->dsk.rsz    += curusers->dsk.rio;
-		curusers->dsk.wsz    +=	curusers->dsk.wio;
-
-		curusers->net.tcpsnd += (*curprocs)->net.tcpsnd;
-		curusers->net.tcprcv += (*curprocs)->net.tcprcv;
-		curusers->net.udpsnd += (*curprocs)->net.udpsnd;
-		curusers->net.udprcv += (*curprocs)->net.udprcv;
-
-		curusers->net.tcpssz += (*curprocs)->net.tcpssz;
-		curusers->net.tcprsz += (*curprocs)->net.tcprsz;
-		curusers->net.udpssz += (*curprocs)->net.udpssz;
-		curusers->net.udprsz += (*curprocs)->net.udprsz;
-
-		if ((*curprocs)->gen.state != 'E')
-		{
-			curusers->mem.vmem   += (*curprocs)->mem.vmem;
-			curusers->mem.rmem   += (*curprocs)->mem.rmem;
-			curusers->mem.pmem   += (*curprocs)->mem.pmem;
-			curusers->mem.vlibs  += (*curprocs)->mem.vlibs;
-			curusers->mem.vdata  += (*curprocs)->mem.vdata;
-			curusers->mem.vstack += (*curprocs)->mem.vstack;
-			curusers->mem.vswap  += (*curprocs)->mem.vswap;
-			curusers->mem.rgrow  += (*curprocs)->mem.rgrow;
-			curusers->mem.vgrow  += (*curprocs)->mem.vgrow;
-		}
+		accumulate(*curprocs, curusers);
 	}
 
 	if (curusers->gen.pid)
@@ -2055,15 +2210,15 @@ cumusers(struct tstat **curprocs, struct tstat *curusers, int numprocs)
 	return numusers;
 }
 
+
 /*
 ** accumulate all processes with the same name (i.e. same program)
 ** into a new list
 */
 static int
-cumprocs(struct tstat **curprocs, struct tstat *curprogs, int numprocs)
+cumprogs(struct tstat **curprocs, struct tstat *curprogs, int numprocs)
 {
 	register int	i, numprogs;
-	count_t		nett_wsz;
 
 	/*
 	** sort list of active processes in order of process-name
@@ -2078,7 +2233,7 @@ cumprocs(struct tstat **curprocs, struct tstat *curprogs, int numprocs)
 		if (procsuppress(*curprocs, &procsel))
 			continue;
 
-		if ((*curprocs)->gen.state == 'E' && supexits)
+		if ((*curprocs)->gen.state == 'E' && suppressexit)
 			continue;
 
 		if ( strcmp(curprogs->gen.name, (*curprocs)->gen.name) != 0)
@@ -2091,46 +2246,7 @@ cumprocs(struct tstat **curprocs, struct tstat *curprogs, int numprocs)
 			strcpy(curprogs->gen.name, (*curprocs)->gen.name);
 		}
 
-		curprogs->gen.pid++;		/* misuse as counter */
-
-		curprogs->gen.isproc  = 1;
-		curprogs->gen.nthr   += (*curprocs)->gen.nthr;
-		curprogs->cpu.utime  += (*curprocs)->cpu.utime;
-		curprogs->cpu.stime  += (*curprocs)->cpu.stime;
-
- 		if ((*curprocs)->dsk.wsz > (*curprocs)->dsk.cwsz)
-                	nett_wsz = (*curprocs)->dsk.wsz -(*curprocs)->dsk.cwsz;
-		else
-			nett_wsz = 0;
-
-		curprogs->dsk.rio    += (*curprocs)->dsk.rsz;
-		curprogs->dsk.wio    += nett_wsz;
-			
-		curprogs->dsk.rsz    += curprogs->dsk.rio;
-		curprogs->dsk.wsz    +=	curprogs->dsk.wio;
-			
-		curprogs->net.tcpsnd += (*curprocs)->net.tcpsnd;
-		curprogs->net.tcprcv += (*curprocs)->net.tcprcv;
-		curprogs->net.udpsnd += (*curprocs)->net.udpsnd;
-		curprogs->net.udprcv += (*curprocs)->net.udprcv;
-
-		curprogs->net.tcpssz += (*curprocs)->net.tcpssz;
-		curprogs->net.tcprsz += (*curprocs)->net.tcprsz;
-		curprogs->net.udpssz += (*curprocs)->net.udpssz;
-		curprogs->net.udprsz += (*curprocs)->net.udprsz;
-
-		if ((*curprocs)->gen.state != 'E')
-		{
-			curprogs->mem.vmem   += (*curprocs)->mem.vmem;
-			curprogs->mem.rmem   += (*curprocs)->mem.rmem;
-			curprogs->mem.pmem   += (*curprocs)->mem.pmem;
-			curprogs->mem.vlibs  += (*curprocs)->mem.vlibs;
-			curprogs->mem.vdata  += (*curprocs)->mem.vdata;
-			curprogs->mem.vstack += (*curprocs)->mem.vstack;
-			curprogs->mem.vswap  += (*curprocs)->mem.vswap;
-			curprogs->mem.rgrow  += (*curprocs)->mem.rgrow;
-			curprogs->mem.vgrow  += (*curprocs)->mem.vgrow;
-		}
+		accumulate(*curprocs, curprogs);
 	}
 
 	if (curprogs->gen.pid)
@@ -2139,6 +2255,110 @@ cumprocs(struct tstat **curprocs, struct tstat *curprogs, int numprocs)
 	return numprogs;
 }
 
+/*
+** accumulate all processes per container in new list
+*/
+static int
+cumconts(struct tstat **curprocs, struct tstat *curconts, int numprocs)
+{
+	register int	i, numconts;
+
+	/*
+	** sort list of active processes in order of container (increasing)
+	*/
+	qsort(curprocs, numprocs, sizeof(struct tstat *), compcon);
+
+	/*
+	** accumulate all processes per container in the new list
+	*/
+	for (numconts=i=0; i < numprocs; i++, curprocs++)
+	{
+		if (procsuppress(*curprocs, &procsel))
+			continue;
+
+		if ((*curprocs)->gen.state == 'E' && suppressexit)
+			continue;
+ 
+		if ( strcmp(curconts->gen.container,
+                         (*curprocs)->gen.container) != 0)
+		{
+			if (curconts->gen.pid)
+			{
+				numconts++;
+				curconts++;
+			}
+			strcpy(curconts->gen.container,
+			    (*curprocs)->gen.container);
+		}
+
+		accumulate(*curprocs, curconts);
+	}
+
+	if (curconts->gen.pid)
+		numconts++;
+
+	return numconts;
+}
+
+
+/*
+** accumulate relevant counters from individual task to
+** combined task
+*/
+static void
+accumulate(struct tstat *curproc, struct tstat *curstat)
+{
+	count_t		nett_wsz;
+
+	curstat->gen.pid++;		/* misuse as counter */
+
+	curstat->gen.isproc  = 1;
+	curstat->gen.nthr   += curproc->gen.nthr;
+	curstat->cpu.utime  += curproc->cpu.utime;
+	curstat->cpu.stime  += curproc->cpu.stime;
+
+	if (curproc->dsk.wsz > curproc->dsk.cwsz)
+               	nett_wsz = curproc->dsk.wsz -curproc->dsk.cwsz;
+	else
+		nett_wsz = 0;
+
+	curstat->dsk.rio    += curproc->dsk.rsz;
+	curstat->dsk.wio    += nett_wsz;
+
+	curstat->dsk.rsz     = curstat->dsk.rio;
+	curstat->dsk.wsz     = curstat->dsk.wio;
+
+	curstat->net.tcpsnd += curproc->net.tcpsnd;
+	curstat->net.tcprcv += curproc->net.tcprcv;
+	curstat->net.udpsnd += curproc->net.udpsnd;
+	curstat->net.udprcv += curproc->net.udprcv;
+
+	curstat->net.tcpssz += curproc->net.tcpssz;
+	curstat->net.tcprsz += curproc->net.tcprsz;
+	curstat->net.udpssz += curproc->net.udpssz;
+	curstat->net.udprsz += curproc->net.udprsz;
+
+	if (curproc->gen.state != 'E')
+	{
+		if (curstat->mem.pmem != -1)
+		{
+			if  (curproc->mem.pmem != -1)  // no errors?
+				curstat->mem.pmem += curproc->mem.pmem;
+			else
+				curstat->mem.pmem  = -1;
+		}
+
+		curstat->mem.vmem   += curproc->mem.vmem;
+		curstat->mem.rmem   += curproc->mem.rmem;
+		curstat->mem.vlibs  += curproc->mem.vlibs;
+		curstat->mem.vdata  += curproc->mem.vdata;
+		curstat->mem.vstack += curproc->mem.vstack;
+		curstat->mem.vswap  += curproc->mem.vswap;
+		curstat->mem.rgrow  += curproc->mem.rgrow;
+		curstat->mem.vgrow  += curproc->mem.vgrow;
+	}
+}
+
 
 /*
 ** function that checks if the current process is selected or suppressed;
@@ -2213,6 +2433,24 @@ procsuppress(struct tstat *curstat, struct pselection *sel)
 		}
 	}
 
+	/*
+	** check if only processes related to a particular container
+	** should be shown (container 'H' stands for native host processes)
+	*/
+	if (sel->container[0])
+	{
+		if (sel->container[0] == 'H')	// only host processes
+		{
+			if (curstat->gen.container[0])
+				return 1;
+		}
+		else
+		{
+			if (memcmp(sel->container, curstat->gen.container, 12))
+				return 1;
+		}
+	}
+
 	return 0;
 }
 
@@ -2397,6 +2635,10 @@ generic_init(void)
 			showtype  = MCUMPROC;
 			break;
 
+		   case MCUMCONT:
+			showtype  = MCUMCONT;
+			break;
+
 		   case MSYSFIXED:
 			if (fixedhead)
 				fixedhead=0;
@@ -2426,10 +2668,10 @@ generic_init(void)
 			break;
 
 		   case MSUPEXITS:
-			if (supexits)
-				supexits = 0;
+			if (suppressexit)
+				suppressexit = 0;
 			else
-				supexits = 1;
+				suppressexit = 1;
 			break;
 
 		   case MCOLORS:
@@ -2536,15 +2778,17 @@ static struct helptext {
 	{"\t'%c'  - total resource consumption per user\n", 	MCUMUSER},
 	{"\t'%c'  - total resource consumption per program (i.e. same "
 	 "process name)\n",					MCUMPROC},
+	{"\t'%c'  - total resource consumption per container\n",MCUMCONT},
 	{"\n",							' '},
 	{"Process selections (keys shown in header line):\n",	' '},
 	{"\t'%c'  - focus on specific user name           "
 	                              "(regular expression)\n", MSELUSER},
-	{"\t'%c'  - focus on specific process name        "
+	{"\t'%c'  - focus on specific program name        "
 	                              "(regular expression)\n", MSELPROC},
+	{"\t'%c'  - focus on specific contained id (CID)\n",    MSELCONT},
 	{"\t'%c'  - focus on specific command line string "
 	                              "(regular expression)\n", MSELARG},
-	{"\t'%c'  - focus on specific process-id (PID)\n",      MSELPID},
+	{"\t'%c'  - focus on specific process id (PID)\n",      MSELPID},
 	{"\n",							' '},
 	{"System resource selections (keys shown in header line):\n",' '},
 	{"\t'%c'  - focus on specific system resources    "
@@ -2566,7 +2810,7 @@ static struct helptext {
 		 						MTHREAD},
 	{"\t'%c'  - show all processes (default: active processes) (toggle)\n",
 								MALLPROC},
-	{"\t'%c'  - show fixed number of header-lines              (toggle)\n",
+	{"\t'%c'  - show fixed number of header lines              (toggle)\n",
 								MSYSFIXED},
 	{"\t'%c'  - suppress sorting system resources              (toggle)\n",
 								MSYSNOSORT},
@@ -2586,20 +2830,20 @@ static struct helptext {
 	{"\t'%c'  - rewind to begin of raw file\n",		MRESET},
 	{"\n",							' '},
 	{"Miscellaneous commands:\n",				' '},
-	{"\t'%c'  - change interval-timer (0 = only manual trigger)\n",
+	{"\t'%c'  - change interval timer (0 = only manual trigger)\n",
 								MINTERVAL},
 	{"\t'%c'  - manual trigger to force next sample\n",	MSAMPNEXT},
 	{"\t'%c'  - reset counters to boot time values\n",	MRESET},
-	{"\t'%c'  - pause-button to freeze current sample (toggle)\n",
+	{"\t'%c'  - pause button to freeze current sample (toggle)\n",
 								MPAUSE},
 	{"\n",							' '},
 	{"\t'%c'  - limited lines for per-cpu, disk and interface resources\n",
 								MSYSLIMIT},
 	{"\t'%c'  - kill a process (i.e. send a signal)\n",	MKILLPROC},
 	{"\n",							' '},
-	{"\t'%c'  - version-information\n",			MVERSION},
-	{"\t'%c'  - help-information\n",			MHELP1},
-	{"\t'%c'  - help-information\n",			MHELP2},
+	{"\t'%c'  - version information\n",			MVERSION},
+	{"\t'%c'  - help information\n",			MHELP1},
+	{"\t'%c'  - help information\n",			MHELP2},
 	{"\t'%c'  - quit this program\n",			MQUIT},
 };
 
@@ -2721,8 +2965,10 @@ generic_usage(void)
 	printf("\t  -%c  show cumulated process-info per user\n",
 			MCUMUSER);
 	printf("\t  -%c  show cumulated process-info per program "
-	                "(i.e. same name)\n\n",
+	                "(i.e. same name)\n",
 			MCUMPROC);
+	printf("\t  -%c  show cumulated process-info per container\n\n",
+			MCUMCONT);
 	printf("\t  -%c  sort processes in order of cpu-consumption "
 	                "(default)\n",
 			MSORTCPU);
@@ -3001,6 +3247,10 @@ do_flags(char *name, char *val)
 			showtype  = MCUMPROC;
 			break;
 
+		   case MCUMCONT:
+			showtype  = MCUMCONT;
+			break;
+
 		   case MALLPROC:
 			deviatonly = 0;
 			break;
@@ -3030,7 +3280,7 @@ do_flags(char *name, char *val)
 			break;
 
 		   case MSUPEXITS:
-			supexits = 1;
+			suppressexit = 1;
 			break;
 		}
 	}
