@@ -498,6 +498,7 @@ sys_printdef *netintfsyspdefs[] = {
         0
 };
 
+
 /*
  * table with all proc_printdefs
  */
@@ -531,6 +532,7 @@ proc_printdef *allprocpdefs[]=
 	&procprt_FSGID,
 	&procprt_CTID,
 	&procprt_VPID,
+	&procprt_CID,
 	&procprt_STDATE,
 	&procprt_STTIME,
 	&procprt_ENDATE,
@@ -569,6 +571,18 @@ proc_printdef *allprocpdefs[]=
         0
 };
 
+/*
+ * table with all proc_printdefs with PID/TID width to be initialized
+ */
+proc_printdef *idprocpdefs[]= 
+{
+	&procprt_PID,
+	&procprt_TID,
+	&procprt_PPID,
+	&procprt_VPID,
+	0
+};
+
 
 /***************************************************************/
 /*
@@ -586,6 +600,7 @@ proc_printpair cmdprocs[MAXITEMS];
 proc_printpair ownprocs[MAXITEMS];
 proc_printpair totusers[MAXITEMS];
 proc_printpair totprocs[MAXITEMS];
+proc_printpair totconts[MAXITEMS];
 
 
 /*****************************************************************/
@@ -729,6 +744,49 @@ make_sys_prints(sys_printpair *ar, int maxn, const char *pairs,
 }
 
 
+
+/*
+ * init_proc_prints: determine width of columns that are
+ *                   dependent of dynamic values 
+ */
+void 
+init_proc_prints()
+{
+	int 	i, numdigits = 5;
+	char	linebuf[64];
+	FILE	*fp;
+
+	/*
+	** determine maximum number of digits for PID/TID
+	*/
+	if ( (fp = fopen("/proc/sys/kernel/pid_max", "r")) != NULL)
+	{
+                if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
+                {
+                        numdigits = strlen(linebuf) - 1;
+                }
+
+                fclose(fp);
+        }
+
+	/*
+	** fill number of digits for various PID/TID columns
+	** and reformat header to new width
+	*/
+	for (i=0; idprocpdefs[i] != 0; i++)
+	{
+		idprocpdefs[i]->width = numdigits;
+
+		if ( strlen(idprocpdefs[i]->head) < numdigits)
+		{
+			char *p = malloc(numdigits) + 1;
+
+			sprintf(p, "%*s", numdigits, idprocpdefs[i]->head);
+			idprocpdefs[i]->head = p;
+		}
+	}
+}
+
 /*
  * make_proc_prints: make array of proc_printpairs
  * input: string, proc_printpair array, maxentries
@@ -822,9 +880,7 @@ totalcap(struct syscap *psc, struct sstat *sstat,
 ** calculate cumulative system- and user-time for all active processes
 */
 void
-pricumproc(struct sstat *sstat, struct tstat **proclist,
-           int nactproc, int ntask, int totproc,
-	   int totrun, int totslpi, int totslpu, int totzomb,
+pricumproc(struct sstat *sstat, struct devtstat *devtstat,
            int nexit, unsigned int noverflow, int avgval, int nsecs)
 {
 
@@ -1076,19 +1132,19 @@ pricumproc(struct sstat *sstat, struct tstat **proclist,
         extraparam extra;
 
 
-        for (i=0, extra.totut=extra.totst=0; i < nactproc; i++)
+        for (i=0, extra.totut=extra.totst=0; i < devtstat->nprocactive; i++)
         {
-		struct tstat *curstat = *(proclist+i);
+		struct tstat *curstat = *(devtstat->procactive+i);
 
                 extra.totut	+= curstat->cpu.utime;
                 extra.totst 	+= curstat->cpu.stime;
         }
 
-        extra.nproc	= totproc;
-	extra.ntrun	= totrun;
-	extra.ntslpi	= totslpi;
-	extra.ntslpu	= totslpu;
-        extra.nzomb	= totzomb;
+        extra.nproc	= devtstat->nprocall;
+	extra.ntrun	= devtstat->totrun;
+	extra.ntslpi	= devtstat->totslpi;
+	extra.ntslpu	= devtstat->totslpu;
+        extra.nzomb	= devtstat->totzombie;
         extra.nexit	= nexit;
         extra.noverflow	= noverflow;
         extra.avgval	= avgval;
@@ -1106,7 +1162,7 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
 							char autosort)
 {
         static int      firsttime=1;
-        static int      prev_supportflags = -1;;
+        static int      prev_supportflags = -1, prev_threadview = -1;
 
 	/*
  	** determine once the layout of all per-process reports
@@ -1114,6 +1170,8 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
 	*/
         if (firsttime) 
         {
+		init_proc_prints();
+
                 make_proc_prints(memprocs, MAXITEMS, 
                         "PID:10 TID:3 MINFLT:2 MAJFLT:2 VSTEXT:4 VSLIBS:4 "
 			"VDATA:4 VSTACK:4 VSIZE:6 RSIZE:7 PSIZE:5 "
@@ -1122,9 +1180,9 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
                         "built-in memprocs");
 
                 make_proc_prints(schedprocs, MAXITEMS, 
-                        "PID:10 TID:6 VPID:5 CTID:5 TRUN:7 TSLPI:7 TSLPU:7 "
-			"POLI:8 NICE:9 PRI:9 RTPR:9 CPUNR:8 ST:8 EXC:8 "
-                        "S:8 SORTITEM:10 CMD:10", 
+                        "PID:10 TID:6 CID:5 VPID:4 CTID:4 TRUN:7 TSLPI:7 "
+			"TSLPU:7 POLI:8 NICE:9 PRI:9 RTPR:9 CPUNR:8 ST:8 "
+			"EXC:8 S:8 SORTITEM:10 CMD:10", 
                         "built-in schedprocs");
 
                 make_proc_prints(dskprocs, MAXITEMS, 
@@ -1142,7 +1200,7 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
                         "built-in netprocs");
 
                 make_proc_prints(varprocs, MAXITEMS,
-                        "PID:10 TID:4 PPID:9 VPID:1 CTID:1 "
+                        "PID:10 TID:4 PPID:9 CID:2 VPID:1 CTID:1 "
 			"RUID:8 RGID:8 EUID:5 EGID:4 "
      			"SUID:3 SGID:2 FSUID:3 FSGID:2 "
                         "STDATE:7 STTIME:7 ENDATE:5 ENTIME:5 "
@@ -1164,15 +1222,23 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
                         "RSIZE:8 PSIZE:8 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
 			"RNET:6 SNET:6 SORTITEM:10 CMD:10", 
                         "built-in totprocs");
+
+                make_proc_prints(totconts, MAXITEMS, 
+                        "NPROCS:10 SYSCPU:9 USRCPU:9 VSIZE:6 "
+                        "RSIZE:8 PSIZE:8 SWAPSZ:5 RDDSK:7 CWRDSK:7 "
+			"RNET:6 SNET:6 SORTITEM:10 CID:10", 
+                        "built-in totconts");
         }
 
 	/*
  	** update the generic report if needed
 	*/
-	if (prev_supportflags != supportflags)
+	if (prev_supportflags != supportflags || prev_threadview != threadview)
 	{
 		make_proc_dynamicgen();
+
 		prev_supportflags = supportflags;
+		prev_threadview   = threadview;
 
 		if (*showtype == MPROCNET && !(supportflags&NETATOP) )
 		{
@@ -1225,56 +1291,73 @@ priphead(int curlist, int totlist, char *showtype, char *showorder,
            case MCUMPROC:
                 showhdrline(totprocs, curlist, totlist, *showorder, autosort);
                 break;
+
+           case MCUMCONT:
+                showhdrline(totconts, curlist, totlist, *showorder, autosort);
+                break;
         }
 }
 
+/*
+** assemble the layout of the generic line,
+** depending on the supported features (like
+** I/O stats, network stats) and current view
+*/
+#define	FORMPID	"PID:10 "
+#define	FORMTID	"TID:6 "
+#define	FORMCID	"CID:5 "
+#define	FORMCPU	"SYSCPU:9 USRCPU:9 "
+#define FORMMEM	"VGROW:8 RGROW:8 "
+#define FORMDSK	"RDDSK:7 CWRDSK:7 "
+#define FORMNET	"RNET:6 SNET:6 "
+#define FORMMSC	"RUID:3 EUID:2 ST:4 EXC:4 THR:4 S:4 CPUNR:4 "
+#define FORMEND	"SORTITEM:10 CMD:10"
+
 static void
 make_proc_dynamicgen()
 {
-	if ( (supportflags & (IOSTAT|NETATOP)) == (IOSTAT|NETATOP)) 
+	char format[300], *p = format;
+
+	memcpy(p, FORMPID, sizeof FORMPID -1);
+	p += sizeof FORMPID -1;
+
+	if (threadview)
 	{
-		// iostat and netatop data is available
-		make_proc_prints(genprocs, MAXITEMS, 
-			"PID:10 TID:4 SYSCPU:9 USRCPU:9 "
-			"VGROW:8 RGROW:8 "
-			"RDDSK:7 CWRDSK:7 "
-			"RNET:6 SNET:6 S:5 "
-			"SORTITEM:10 CMD:10", 
-			"built-in genprocs");
+		memcpy(p, FORMTID, sizeof FORMTID -1);
+		p += sizeof FORMTID -1;
 	}
-	else if (supportflags & IOSTAT) 
+
+	if (supportflags & DOCKSTAT)
 	{
-		// only iostat data is available
-		make_proc_prints(genprocs, MAXITEMS, 
-			"PID:10 TID:4 RUID:3 EUID:2 THR:4 "
-			"SYSCPU:9 USRCPU:9 "
-			"VGROW:8 RGROW:8 "
-			"RDDSK:7 CWRDSK:7 "
-			"ST:6 EXC:6 S:6 "
-			"CPUNR:5 SORTITEM:10 CMD:10", 
-			"built-in genprocs");
-	} 
-	else if (supportflags & NETATOP) 
-	{
-		// only netatop data is available
-		make_proc_prints(genprocs, MAXITEMS, 
-			"PID:10 TID:4 SYSCPU:9 USRCPU:9 "
-			"VGROW:8 RGROW:8 "
-			"RNET:7 SNET:7 "
-			"ST:5 EXC:5 S:6 "
-			"CPUNR:5 SORTITEM:10 CMD:10", 
-			"built-in genprocs");
+		memcpy(p, FORMCID, sizeof FORMCID -1);
+		p += sizeof FORMCID -1;
 	}
-	else 
+
+	memcpy(p, FORMCPU, sizeof FORMCPU -1);
+	p += sizeof FORMCPU -1;
+
+	memcpy(p, FORMMEM, sizeof FORMMEM -1);
+	p += sizeof FORMMEM -1;
+
+	if (supportflags & IOSTAT)
 	{
-		// no optional data is available
-		make_proc_prints(genprocs, MAXITEMS, 
-			"PID:10 TID:4 SYSCPU:9 USRCPU:9 "
-			"VGROW:8 RGROW:8 RUID:4 EUID:3 "
-			"THR:7 ST:7 EXC:7 S:7 "
-			"SORTITEM:10 CMD:10", 
-			"built-in genprocs");
+		memcpy(p, FORMDSK, sizeof FORMDSK -1);
+		p += sizeof FORMDSK -1;
 	}
+
+	if (supportflags & NETATOP)
+	{
+		memcpy(p, FORMNET, sizeof FORMNET -1);
+		p += sizeof FORMNET -1;
+	}
+
+	memcpy(p, FORMMSC, sizeof FORMMSC -1);
+	p += sizeof FORMMSC -1;
+
+	memcpy(p, FORMEND, sizeof FORMEND);
+	p += sizeof FORMEND;
+
+	make_proc_prints(genprocs, MAXITEMS, format, "built-in genprocs");
 }
 
 /*
@@ -1425,6 +1508,10 @@ priproc(struct tstat **proclist, int firstproc, int lastproc, int curline,
                    case MCUMPROC:
                         showprocline(totprocs, curstat, perc, nsecs, avgval);
                         break;
+
+                   case MCUMCONT:
+                        showprocline(totconts, curstat, perc, nsecs, avgval);
+                        break;
                 }
 
                 curline++;
@@ -1999,6 +2086,15 @@ compnam(const void *a, const void *b)
         return strcmp(nama, namb);
 }
 
+int
+compcon(const void *a, const void *b)
+{
+        register char *containera = (*(struct tstat **)a)->gen.container;
+        register char *containerb = (*(struct tstat **)b)->gen.container;
+
+       return strcmp(containera, containerb);
+}
+
 int
 cpucompar(const void *a, const void *b)
 {
